/*
 * Arduino code for microstepper-project DDS-board
 * AW2021-09, see https://ohwr.org/project/microstepper
 * 
 *  Copyright (C) 2021  Anders E.E. Wallin
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * 
 * 
 * Written for the following hardware:
 * - Arduino MKR ZERO
 * - Ethernet Shield for MKR ZERO
 * - IO-board for SPI-signals to devices:
 * -- ADF4350 PLL+VCO
 * -- AD9912 DDS (2pcs A and B)
 * 
 */

#include "Arduino.h"
#include "ddslib.h"
#include "plllib.h"
#include "printf_wrapper.h"
#include "httpd.h" // web-server for controlling DDS-board

// SPI pin definitions
// using MKR ZERO with IO-board
const int SPI_SDIO = 1; // D1
const int SPI_SCLK = 0; // D0
const int SPI_CSB_DDSA  = 2; //D2
const int SPI_CSB_DDSB  = 3; //D3
const int SPI_CSB_PLL  = 4; //D4

// DDS-objects
AD9912::DDS ddsA(SPI_SDIO, SPI_CSB_DDSA, SPI_SCLK, 0);
AD9912::DDS ddsB(SPI_SDIO, SPI_CSB_DDSB, SPI_SCLK, 1);
// PLL-object
ADF4350::PLL pll(SPI_SDIO, SPI_CSB_PLL, SPI_SCLK);
// Webserver
HTTPD::httpd* webserver = new HTTPD::httpd(&ddsA, &ddsB, &pll);

void init_pll(ADF4350::PLL* pll) {
  delay(1000);
  Serial.print("init_pll()\n");
  pll->preset1(); // 10MHz input, 1 GHz DDS clock
  delay(1000);
}

void init_DDS(AD9912::DDS* dds) {
    delay(1000);
    dds->reset();
    delay(100);
  
    int id = dds->read_id();
    delay(20);
    serial_printf("DDS id = %d\n", id);
    delay(20);
  
  dds->set_pd( AD9912::PD_HSTL | AD9912::PLL_PD); // no PLL!
  delay(20);
  dds->set_PLL_parameters(AD9912::VCO_AUTORANGE); // not required?
  delay(20);
  dds->update_registers();
  delay(20);
    
  int8_t f[6] =  { (int8_t)0xc6, (int8_t)0x4b, (int8_t)0x37, (int8_t)0x89, (int8_t)0x41, (int8_t)0x00};
  //int8_t fB[6] = { (int8_t)0xc6, (int8_t)0x4b, (int8_t)0x37, (int8_t)0x89, (int8_t)0x41, (int8_t)0x00};
  
  serial_printf("set f = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", f[5],f[4],f[3],f[2],f[1],f[0] );
  // 6-element array
  dds->set_frequency(f);
  delay(20);
  //ddsB.set_frequency(f);
  //delay(20);
  
  int8_t fr[6];
  int mask8 = 0x000000ff;
  
  dds->get_frequency(fr);
  delay(20);
  //serial_printf("ftw_A = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", fr[5],fr[4],fr[3],fr[2],fr[1],fr[0] );
  serial_printf("get ftw = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", fr[5] & mask8,fr[4] & mask8,fr[3] & mask8,fr[2] & mask8,fr[1] & mask8,(fr[0] & mask8) );


}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  delay(1000);
  Serial.print("initializing.\n");

  init_pll(&pll);
  init_DDS(&ddsA);
  init_DDS(&ddsB);
  
  /*
  delay(1000);
  ddsA.reset();
  delay(100);
  ddsB.reset();
  delay(100);
  
  int A_id = ddsA.read_id();
  delay(20);
  Serial.print(A_id);
  Serial.print("\n");
  int B_id = ddsB.read_id();
  Serial.print(B_id);
  Serial.print("\n");

    
  delay(20);
  
  ddsA.set_pd( AD9912::PD_HSTL | AD9912::PLL_PD); // no PLL!
  delay(20);
  ddsA.set_PLL_parameters(AD9912::VCO_AUTORANGE); // not required?
  delay(20);
  ddsA.update_registers();
  delay(20);

  ddsB.set_pd( AD9912::PD_HSTL | AD9912::PLL_PD); // no PLL!
  delay(20);
  ddsB.set_PLL_parameters(AD9912::VCO_AUTORANGE); // not required?
  delay(20);
  ddsB.update_registers();
  delay(20);

  serial_printf( "DDS A PLL_N = %d \n", ddsA.get_PLL_N() );
  serial_printf( "DDS B PLL_N = %d \n", ddsB.get_PLL_N() );
  
  int8_t f[6] =  { (int8_t)0xc6, (int8_t)0x4b, (int8_t)0x37, (int8_t)0x89, (int8_t)0x41, (int8_t)0x00};
  int8_t fB[6] = { (int8_t)0xc6, (int8_t)0x4b, (int8_t)0x37, (int8_t)0x89, (int8_t)0x41, (int8_t)0x00};
  
  serial_printf("f = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", f[5],f[4],f[3],f[2],f[1],f[0] );
  // 6-element array
  ddsA.set_frequency(f);
  delay(20);
  ddsB.set_frequency(f);
  delay(20);
  
  int8_t fr[6];
  int mask8 = 0x000000ff;
  
  ddsA.get_frequency(fr);
  delay(20);
  //serial_printf("ftw_A = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", fr[5],fr[4],fr[3],fr[2],fr[1],fr[0] );
  serial_printf("ftw_A = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", fr[5] & mask8,fr[4] & mask8,fr[3] & mask8,fr[2] & mask8,fr[1] & mask8,(fr[0] & mask8) );

  ddsB.get_frequency(fr);
  delay(20);
  serial_printf("ftw_B = 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n", fr[5] & mask8,fr[4] & mask8,fr[3] & mask8,fr[2] & mask8,fr[1] & mask8,(fr[0] & mask8) );
*/
  delay(1000);
  webserver->init();
  delay(1000);
  
  Serial.print("ready.\n");
}

void loop() {
  // put your main code here, to run repeatedly:
  webserver->available();
}
