"""
    This file is part of the Microstepper project
    https://www.ohwr.org/project/microstepper
    
    Library for microstepper related functions
    
    released under GPLv3 or later license.
    Anders Wallin, 2019-2020
"""
import bigfloat
import numpy
import requests

dds_clk=1.0e9
ftw_length = pow(2,48)

# nominal F1, F2, for N_div=8, M_div=1024
# these FTWs give output closest to reference frequency, i.e. y approx= 0.
F1 = 112535015102874
F2 = 112589990683444


def ftw_dds(f_hz):
    """ return AD9912 DDS Frequency tuning word, given desired frequency in Hz.
        Assumes 1GHz SYSCLK
        
        Only even FTWs possible in practice (i.e. LSB is allways zero)
    """
    with bigfloat.precision(200):
        a = bigfloat.BigFloat.exact(f_hz)
        b = bigfloat.BigFloat.exact(ftw_length)
        c = bigfloat.BigFloat.exact(dds_clk)
        ftw = a*b/c
        ftwi = round_to_even(ftw)
        return int( ftwi )

def hz_dds(ftw):
    """ return DDS output-frequency in Hz, given frequency tuning word
    """
    return dds_clk*ftw/ftw_length

def round_to_even(f):
    """ AD9912 frequency tuning words are ALLWAYS even
        i.e. the LSB is allways zero.
    """
    return round(f / 2.) * 2

def y_out(ftw1, ftw2, N_div=8.0, M_div=1024.0):
    """ given frequency tuning words, compute fractional frequency of microstepper
        F_out / F_REF
    """
    with bigfloat.precision(200):
        F1term = bigfloat.BigFloat.exact(ftw1 / 4.0)
        F2term = bigfloat.BigFloat.exact(ftw2/ (N_div*M_div ))
        Fsum = F1term+F2term
        y = ftw_length* bigfloat.pow( 10.0*(Fsum), -1.0)-1.0
        y18 = y*bigfloat.BigFloat.exact(1e18)
        return y, y18

def f2_from_y(y18, ftw1, N_div=8.0, M_div=1024.0):
    """ given fractional frequency, compute frequency tuning word for DDS2
        y = f_out/f_ref - 1.0
        y=0 corresponds to f_out = f_ref
        
        y18 input is in units of 1e-18
    """
    with bigfloat.precision(200):
        a = bigfloat.BigFloat.exact( pow(2,48) )/ bigfloat.BigFloat.exact( 10.0 )
        y = bigfloat.BigFloat.exact( 1.0 ) + y18 / bigfloat.BigFloat.exact( 1e18 )
        b = bigfloat.BigFloat.exact( 1.0 ) / y 
        c = bigfloat.BigFloat.exact( ftw1 )/ bigfloat.BigFloat.exact( 4.0 )
        d = bigfloat.BigFloat.exact( N_div*M_div )
        ftw2 = d*( a*b - c )
        #print(ftw2)
        #print(" y - 1 = %.3g" %(y-1.0))
        #print(int(ftw2))
        ftw2i = round_to_even(ftw2)
        return int(ftw2i)

def set_DDS(ftw, DDS="A", IP="192.168.1.18"):
    """ use HTTP to set DDS frequency to given frequency tuning word
        DDS index can be 0 or 1
    """
    hexs = split_hex(ftw)
    ftw_string = "%02x:%02x:%02x:%02x:%02x:%02x" % (hexs[0],hexs[1],hexs[2],hexs[3],hexs[4],hexs[5] )
    r = requests.get('http://%s/DDS:%s:FTW:%s' % (IP, DDS, ftw_string))

def split_hex(ftw):
    """ Split FTW into six 2-digit hex words
    """
    mask = 0x0000000000ff
    out=[]
    for n in range(6):
        hexdigits = (ftw & (mask << n*8) )  >> n*8 
        #print("%x " % hexdigits)
        out.append(hexdigits)
    return out
    
