"""
    This file is part of the Microstepper project
    https://www.ohwr.org/project/microstepper
    
    This simple script computes microstepper output frequency
    given DDS frequency tuning words.
    
    released under GPLv3 or later license.
    Anders Wallin, 2019-2020
"""
import bigfloat
import numpy

def ftw_dds(f_hz):
    """ return AD9912 1GHz SYSCLK FTW
    """
    with bigfloat.precision(200):
        a = bigfloat.BigFloat.exact(f_hz)
        b = bigfloat.BigFloat.exact(ftw_length)
        c = bigfloat.BigFloat.exact(dds_clk)
        ftw = a*b/c
        ftwi = round_to_even(ftw)
        return int( ftwi )

def hz_dds(ftw):
    """ return f_out [Hz]
    """
    return dds_clk*ftw/ftw_length

def round_to_even(f):
    """ AD9912 frequency tuning words are ALLWAYS even
        i.e. the LSB is allways zero.
    """
    return round(f / 2.) * 2

def y_out(ftw1, ftw2, N_div=4.0, M_div=1024.0):
    """ given frequency tuning words, compute fractional frequency of microstepper
        F_out / F_REF
    """
    with bigfloat.precision(200):
        F1term = bigfloat.BigFloat.exact(ftw1 / 4.0)
        F2term = bigfloat.BigFloat.exact(ftw2/ (N_div*M_div ))
        Fsum = F1term+F2term
        y = ftw_length* bigfloat.pow( 10.0*(Fsum), -1.0)-1.0
        y18 = y*bigfloat.BigFloat.exact(1e18)
        return y, y18

def f2_from_y(y18, ftw1, N_div=4.0, M_div=1024.0):
    """ given fractional frequency, compute frequency tuning word for DDS2
        y = f_out/f_ref - 1.0
        y=0 corresponds to f_out = f_ref
        
        y18 input is in units of 1e-18
    """
    with bigfloat.precision(200):
        a = bigfloat.BigFloat.exact( pow(2,48) )/ bigfloat.BigFloat.exact( 10.0 )
        y = bigfloat.BigFloat.exact( 1.0 ) + y18 / bigfloat.BigFloat.exact( 1e18 )
        b = bigfloat.BigFloat.exact( 1.0 ) / y 
        c = bigfloat.BigFloat.exact( ftw1 )/ bigfloat.BigFloat.exact( 4.0 )
        d = bigfloat.BigFloat.exact( N_div*M_div )
        ftw2 = d*( a*b - c )
        #print(ftw2)
        #print(" y - 1 = %.3g" %(y-1.0))
        #print(int(ftw2))
        ftw2i = round_to_even(ftw2)
        return int(ftw2i)

dds_clk=1.0e9
ftw_length = pow(2,48)
DDS2_nom = hz_dds( ftw_dds(400e6) ) # nominal frequency of DDS
N_div = 8.0    # first divider 2/4/8
M_div = 1024.0 # second divider 2**10 = 1024
IF = DDS2_nom/(N_div*M_div) # 97656.25 Hz
print("DDS2 nom = ",DDS2_nom)
print("IF2 = %.3f Hz" % IF)
# nominally we get zero output frequency with these 
# tuning-wrods:
F1 = ftw_dds(400e6-4.0*IF)    # DDS1
IF1 = (100e6-hz_dds(F1)/4.0)
F2approx = ftw_dds(N_div*M_div*IF1)       # DDS2
print("F1  ", F1)
F2 = f2_from_y(0, F1, N_div=N_div, M_div=M_div)
print("F2 approx = ", F2approx)
print("F2        = ", F2)
#IF2 = hz_dds(F2)/(N_div*M_div)
#F1 = ftw_dds(400e6-4.0*IF2)    # DDS1

f1_hz = hz_dds(F1)
print("f1_hz = %.12f Hz" % f1_hz)
print("IF1 = %.12f Hz" % (100e6-f1_hz/4.0))
f2_hz = hz_dds(F2)
print("f2_hz = %.12f Hz" % f2_hz)
print("IF2 = %.12f Hz" % (f2_hz/(N_div*M_div)))
print("IF1/IF2 - 1 = %.3g" % ( (100e6-f1_hz/4.0)/(f2_hz/(N_div*M_div)) - 1.0 ) )

print( 40*"-" )
print( "uStep output fractional frequency y" )
print( "FTW2 = %d (constant)" % F2 )
print( "FTW1\t\tdFTW1\ty" )
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F1x = F1+delta
    y1, _ = y_out(F1x,F2, N_div = N_div) 
    ys.append(y1)
    print( "%d\t%d\t%.3g" % ( F1x, delta, y1) )
print( "y change due to 2 steps in FTW1: %.2g" % (min(numpy.diff(ys))) )
print( 40*"-" )
print( "uStep output fractional frequency y" )
print( "FTW1 = %d (constant)" % F1 )
print( "FTW2\t\tdFTW2\ty" )
ys =[]
for delta in [-5,-4,-3,-2,-1,0,1,2,3,4,5]:
    delta=2*delta
    F2x = F2+delta
    y1, _ = y_out(F1,F2x, N_div = N_div) 
    ys.append(y1)
    print( "%d\t%d\t%.3g" % ( F2x, delta, y1) )

print( "y change due to 2 steps in FTW2: %.3g" % (min(numpy.diff(ys))) )
print( 40*"-" )

y18 = 5678
F2 = f2_from_y(y18, F1, N_div=N_div, M_div=M_div)
print("F2 2e-17= ", F2)
yget, yget18 = y_out(F1,F2, N_div=N_div, M_div=M_div) 
print(yget18)
print(" y_out = %.6g" % yget )
print(" y(set)-y_out = %d " % (y18-yget*1e18) )
print(" y(set)-yget18 = %.6f " % (y18-yget18) )
